####################################################################
# Example SVM code for Survey Practice special issue
#
# Example of how one might conduct SVM classification and Logistic 
# regression for the survey response example.  This is not the full 
# replication  code for our analysis.  It is only meant to 
# illustrate the concepts.  If one’s computer has at least four 
# cores, we recommend initializing a parallel environment as below.
# caret's train() command will automatically recognize the parallel
# environment and use it during cross validation.  To run these 
# commands without parallelization, delete the six lines just after 
# “# to run in parallel.” 
#
# Curtis S. Signorino, Antje Kirchner, 10/27/17
####################################################################
rm(list=ls())
library(kernlab)				# SVM package
library(caret)					# needed for training SVM and for 
								# confusionMatrix()

####################################################################
# to run in parallel
####################################################################
library(parallel) 						
library(foreach)						
library(doParallel)						
threads=detectCores()					
cl=makeCluster(threads-1)				
registerDoParallel(cl)					

####################################################################
# load data.  create test and training data sets.
####################################################################
setwd('~/Dropbox/propscores/SVM/replication')  # change to your path
load("SPData2.Rda")
dtrain=subset(SPData2,fold!=11)
dtest=subset(SPData2,fold==11)

####################################################################
# formula:  response variable ~ covariates
####################################################################
f = newrespond20 ~ age + sex  + wborace + hispanic2 + region + educ3 + incgrp4 + telstat + wrkcata  + ratcat2

####################################################################
# svm.  train() and trainControl() are part of the caret package.
####################################################################
ctrl = trainControl(method="cv", number=4, summaryFunction=twoClassSummary, classProbs=TRUE)
m.svm = train(f, data=dtrain,
				 method = "svmRadial",
                 preProc = c("center","scale"),
                 metric="ROC",
                 tuneLength=8,
                 trControl=ctrl)        
m.svm										# print results

p.svm=predict(m.svm, newdata=dtest, type='prob')		# predicted probs
p.svm=p.svm$Respondent
p.svm.01=ifelse(p.svm>=.5, 1, 0)			# predicted outcomes
confusionMatrix(p.svm.01, ifelse(dtest$newrespond20=="Respondent",1,0), positive="1")

####################################################################
# logistic regression
####################################################################
m.logit = glm(f, data=dtrain, family=binomial(logit))
p.logit = predict(m.logit, newdata=dtest, type='response')  # predicted probs
p.logit.01 = ifelse(p.logit>=.5, 1, 0)		# predicted outcomes
confusionMatrix(p.logit.01, ifelse(dtest$newrespond20=="Respondent",1,0), positive="1")

stopCluster(cl)
